program px2js;
{-- Delphi program for conversion of a PC-AXIS-fil to a Javascript version
    Bo Justusson, Statistics Sweden, 2005-04-07
    2008-02-13 Replaced read-file of start javascript-code with text-strings.
    2005-08-25 All files in same directory (before /px into /js).
    2005-08-23 Added keyword NVAR=x
    To do: use DIMENSION or PXDIM instead of VAR.
    To do: Calculate NVAR first, then write header with array declaration (now include file).
    
    Single valued keywords:  only add PX. e.g. PX.MATRIX="BE0101A1";

    Keyword names: Uppercase and replace '-' with '_'

    Variable definitions e.g. STUB="region","crop";  HEADING="time";
       translate into:
          PX.VAR[0].NAME="region";  PX.VAR[1].NAME="crop";  PX.VAR[2].NAME="time";
          and  PX.STUB=0,1;  PX.HEADING=2;   PX.NVAR=3;  ( =PX.VAR.Length(); ?)

    Arrays VALUES("region")="Stockholm","Uppsala",....;
           CODES("region")="01","03",...;
       translate into PX.VAR[0].VALUES=["Stockholm","Uppsala",...];
                      PX.VAR[0].CODES=["01","03",...];
      ('VAR' or 'DIM' ??  Need java-script Array def??)

    Multi-line strings, e.g. NOTE="text..." (new line) "more text...";
    translate into  PX.NOTE="text..."+ (new line) "more text...";

    Values for keywords are strings ("val" or 'val'), also for YES, NO, decimals etc.
    Values in DATA are numbers (123..) or strings e.g. ".." .
--}
{$APPTYPE CONSOLE}
uses
  SysUtils; {for some string functions}

{-- String functions used:
    Pos(s1, s) first position for s1 in s (=1,2,... and =0 if not found)
    Length(s)
    SubStr(s,pos1,pos2) returns substring of s from pos1 to pos2.
    StrSepToComma(s,sep) replace all sep by commas (javascript's array separator)
    StrBlip(s) adds double-blips around string, if missing.
-----}
function SubStr( s:String; pos1,pos2:Integer): String;
   begin if (pos2>=pos1) then SubStr := Copy( s, pos1, pos2-pos1+1)
                         else SubStr := '';  end;

function StrSepToComma(s, sep: String) : String;
   begin StrSepToComma := StringReplace(s, sep, ',', [rfReplaceAll] ); end;

function StrBlip(s:String):String;
   {-- adds blips to strings: abc to "abc" and abc; to "abc";  ---}
   var len:Integer;
   begin s:=Trim(s);  len:=Length(s);
         if      (s='')   then s:=''
         else if (s='"')  then s:=''
         else if (s=';')  then s:=';'
         else if (s='";') then s:=';'
         else begin
            if (s[1]<>'"') then begin s := '"' + s ; len:=Length(s); end;
            if ((s[len]=';') and (s[len-1]<>'"'))
               then begin s := SubStr(s,1,len-1) + '";' ; len:=Length(s); end;
            if ((s[len]<>';') and (s[len]<>'"'))
               then s := s + '"' ;
         end;
         StrBlip:=s;
   end;


const blip = #39; // '
var searchRec : TSearchRec; //response in FindFirst/Next files '*.px' -> .name.
    fIn, fOut : TextFile;
    searchStr, sInfileName, sOutfileName : String;
    sIn, sKeyword, sKeywordExpr, sVarname, sRest, sOut, sep, plus : String;
    Nh, searchRet, Nfile, p1,p2,p3,p4,p5, len1, len2, lenR, Nvar, Nstub, Nheading, i, Nline : Integer;
    varFound : Boolean;
    varname: array[0..10] of String;
    sJSHeader: array[0..10] of String;

function GetVarNr(s:String):Integer;
   {-- Checks if s contains some VAR[i].NAME then returns i, else -1 ---}
   var i,vnr:Integer;
   begin
      vnr:=-1;
      for i:=0 to Nvar-1 do
      begin  if (Pos(varname[i], s) > 0 ) then vnr:=i;  end;
      GetVarNr:=vnr;
   end;

label loopend, endlabel;
begin
   Nfile:=0;
   searchStr:='*.px';
   searchRet := FindFirst(searchStr, faAnyFile, searchRec);
   if (searchRet<>0) then goto endlabel;

{2008-02-13 Replaced read-file with text-strings -----
   AssignFile(fIn, 'jsheader.txt');
   Reset(fIn);  Nh:=0;
   repeat
      Nh:=Nh+1; Readln(fIn, sJSHeader[Nh]);
   until SeekEof(fIn);
   Close(fIn);
------------------------------------------------------}
sJSHeader[1]:='/* JavaScript version of PC-AXIS file. Created with PX2JS.exe from Statistics Sweden (Bo Justusson). */ ';
sJSHeader[2]:='var VAR = new Array(10); ';
sJSHeader[3]:='function PXVAR(n,v,c,d,e) {this.NAME=n; this.VALUES=v; this.CODES=c; this.DOMAIN=d; this.ELIMINATION=e; } ';
sJSHeader[4]:='for (i=0;i<10;i++) { VAR[i] = new PXVAR('''','''','''','''',''''); } ';
sJSHeader[5]:='/***************************************************************/ ';
Nh:=5;          { note in a string '' =simple '  and '''' = ''   !!!}

 repeat //loop over input px-files.
   Nfile := Nfile+1;
   Nvar:=0;     Nline:=0;    // Nstub:=0;   Nheading:=0;
   sInfileName  := searchRec.name; // 'c:\data\px2js\AM0207A3tid.px';
   sOutfileName := sInfileName + '.js'; {2005-08-25}
   {--sOutfileName := '..\js\'+ sInfileName + '.js';--}
   Writeln('IN: ' + sInfileName  ); {2008-02-13}

   AssignFile(fIn, sInfileName);
   Reset(fIn);
   AssignFile(fOut, sOutfileName);
   Rewrite(fOut);
   for i:=1 to Nh do Writeln(fOut, sJSHeader[i]);  //write JS-header.
        {---- Keyword Cases:
              Keyword names: replace '-' with '_' , and all with UPPER_CASE letters.
              STUB, HEADING - Variable definitions
                 get var-numbers i=0,1,.. (VAR[i])    (or 'idnr' , 'i' is too simple)
                 and set name-value VAR[i].name="varname".
                 STUB=1,2; HEADING=3; i.e. varnr-array.
              CODES, VALUES, TIMEVAL - Array values (i=0,1,..,N-1 in javascript)
                 determine correct Var-number and
                 insert leading '[' and trailing ']' - no other changes!
              DATA
                 change separator to ','  and add leading '[' and trailing ']'
              VALUENOTE, CELLNOTE not handled yet.
              All others (single valued, can be text-string on several lines.
                  insert '+' at start of each new line.
              MAP and TIMEVAL
                 MAP.VARNR and MAP.VAL e.g.="Sweden_municipality"; )
                 TIME.VARNR and TIME.VAL e.g.='TLIST(A1),"1999","2000",...';
                     (= 'in-array'; - with simple blips).
           -----}
   Repeat
       Readln(fIn, sIn);      Nline:=Nline+1;
       p1 := Pos('=' , sIn);
       len1 := System.Length(sIn);

       if (p1>0) then begin  // keyword found
          sKeywordExpr := Trim(System.Copy(sIn, 1, p1-1));
          // Check if variable keyword type CODES("region")
          p2 := Pos( '(' , sKeywordExpr);
          p3 := Pos( ')' , sKeywordExpr);
          if (p2>0) then begin
             sKeyword := System.Copy(sKeyWordExpr, 1, p2-1);
             // javascript is case-sensitive, so always upper-case:
             sKeyword := UpperCase(sKeyword);
             sVarname := SubStr(sKeyWordExpr, p2+1, p3-1); // e.g. "region" .
          end
          else begin sKeyword:=sKeywordExpr; sVarname:=''; end;
          // Replace '-' in kywords with '_'
          p4 := Pos('-' , sKeyword);
          if (p4>0) then sKeyword[p4] := '_';

          if (sKeyword = 'DATA') then goto loopend;


          if ((sKeyword='STUB') or (sKeyword='HEADING'))
          then begin // output e.g. VAR[0].NAME="region";
             sRest := SubStr(sIn, p1+1, len1);
          repeat
             lenR := Length(sRest);
             Nvar:=Nvar+1; // 1,2,...
             p4:=Pos('"', sRest);
             p5:=Pos('","', sRest);
             if (p5=0) then p5:=lenR-1; //last var.

             varname[Nvar-1] := SubStr(sRest,p4,p5);
             sOut := 'VAR[' +IntToStr(Nvar-1) + '].NAME=' +varname[Nvar-1] +';';
             Writeln(fOut, sOut);

             //prepare for next variable
             if (p5 < (lenR-1)) then
                sRest := SubStr(sRest, p5+2, lenR );
           until ((p5 >= (lenR-1)) or (Nvar>20));
          end

          else if ((sKeyword='CODES') or (sKeyword='VALUES'))
          then begin //array values, add leading '[' and trailing '];'
              i := GetVarNr(sVarname);
              if (i>=0)
              then begin
                    if (sIn[len1]=';')
                       then sRest := SubStr(sIn, p1+1, len1-1) + '];'
                       else sRest := SubStr(sIn, p1+1, len1);
                    sOut := 'VAR[' +IntToStr(i)+ '].' +sKeyword+ '=[' +sRest;
                    Writeln(fOut,sOut);
                    if (sIn[len1]<>';') then
                    repeat //read more lines
                       Readln(fIn, sRest); Nline:=Nline+1;
                       lenR:=Length(sRest);
                       if (sRest[lenR]=';')
                          then sOut := SubStr(sRest, 1, lenR-1) + '];'
                          else sOut := sRest;
                       Writeln(fOut, sOut);
                    until ((srest[lenR]=';') or (Nline>10000)) ;
              end;
          end

          else begin //single valued keyword, can be string on several lines.
             sRest := SubStr(sIn, p1+1, len1);
             lenR  := Length(sRest);

             if (sKeyword='TIMEVAL') //Special: Copy value and add blips(').
                 then begin
                    sOut:='TIMEVAL='+blip+SubStr(sRest,1,lenR-1)+blip+';'  ;
                    Writeln(fOut,sOut);
                    goto loopend;
                 end;

             i := GetVarNr(sVarname); //check if some varname matches sVarname
             if (i>=0)
                then begin sOut := 'VAR[' +IntToStr(i) + '].' +sKeyword+'='; end
                else sOut := sKeyword + '=';

             Write(fOut,sOut); //Write keyword= , no end-of-line.
             if (Length(sRest) = 0)
                then plus:=''
                else begin
                   sRest := StrBlip(sRest);
                   Writeln(fOut, sRest);   plus:='+';
                end;

             //check if more lines to output.
             lenR:=Length(sRest);
             if ((lenR=0) or (sRest[lenR] <> ';') ) then
             repeat
                Readln(fIn, sRest);     Nline:=Nline+1;
                sRest := StrBlip(sRest);
                lenR  := Length(sRest);
                if (lenR <=1 ) then plus:=''; //only sRest=';'
                Writeln(fOut, plus + sRest);
                plus:='+';
             until ((sRest[lenR]=';') or (Nline>1000)); //Nline>1000 safety!
           end;


       end // end of keyword found
       else begin
          //sOut := sIn;
       end;
       //Writeln(fOut, sOut);
loopend:
   until (sKeyword='DATA') or (Nline>1000); //checked at start of loop and jumps here.

   sOut := 'NVAR=' +IntToStr(Nvar) + ';' ;  {2005-08-25}
   Writeln(fOut, sOut);                     {2005-08-25}

   //--- DATA section ---  DATA = [ 12, 23, 34, "..", 56, .... ];
   Writeln(fOut, 'DATA=[' );

   Nline:=0;
   sep:=' ';  //separator.
   sRest := Trim(SubStr( sIn, p1+1, len1));
   sOut  := StrSeptoComma(sRest, sep); //
   if (Length(sOut)>1)
      then begin Nline:=1; Writeln(fOut, sOut); end;

   while (not Eof(fIn)) do
      begin
        Readln(fIn, sIn);
        sIn  := Trim(sIn);    //trims away trailing blank
        len1 := Length(sIn);
        sOut := StrSepToComma(sIn, sep);
        len2 := Length(sOut);

        if ((Nline>0) and (len2>1))
           then sOut := ','+sOut; //add leading comma on new lines
        len2:=Length(sOut);

        if ((len2>0) and (sOut[len2]=';'))     // remove ending semi-colon
           then  sOut:=SubStr(sOut,1,len2-1);
        len2:=Length(sOut);

        if (len2>0)
           then begin Writeln(fOut, sOut);  Nline := Nline+1; end;
      end;
   Writeln(fOut, '];' );  // adds ending bracket and semi-colon.
   Close(fIn);
   Close(fOut);

   searchRet := FindNext(searchRec);
   until ((searchRet<>0) or (Nfile>10000)); //of Find *.px-files

endlabel:
   Writeln('Nfiles: '+IntToStr(Nfile)+ ' Press Enter');    {2008-02-13}
   Readln;
end.